/*
 *  (C) Copyright 2001 Mulle kybernetiK. All rights reserved.
 *
 *  Permission to use, copy, modify and distribute this software and its
 *  documentation is hereby granted, provided that both the copyright notice
 *  and this permission notice appear in all copies of the software, derivative
 *  works or modified versions, and any portions thereof, and that both notices
 *  appear in supporting documentation, and that credit is given to
 *  Mulle kybernetiK in all documents and publicity pertaining to direct or
 *  indirect use of this code or its derivatives.
 *
 *  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH
 *  MAY HAVE SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS
 *  SOFTWARE IN ITS "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY
 *  LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING DIRECTLY OR
 *  INDIRECTLY FROM THE USE OF THIS SOFTWARE OR OF ANY DERIVATIVE WORK.
 *
 *  Coded by Nat!
 *
 *  $Id$
 *
 *  $Log$
 */
#import "Controller.h"
#import "Stopwatch.h"
#import "PrivateMutableArray.h"
#import "PrivateUnsafeMutableArray.h"
#import "PrivateInlineRefCountingString.h"


#define TESTFILE    @"/Developer/Documentation/ReleaseNotes/AppKitOlderNotes.html"
#define LOOPS       3000


@implementation Controller


static NSMutableArray  *unoptimized( NSArray *lines)
{
   NSMutableArray      *result;
   NSString            *s;
   NSEnumerator        *rover;
   
   result = [NSMutableArray arrayWithCapacity:[lines count]];
   rover  = [lines reverseObjectEnumerator];
   
   while( s = [rover nextObject])
      [result addObject:s];
   
   return( result);
}


static NSMutableArray  *imped( NSArray *lines)
{
   NSMutableArray      *result;
   NSString            *s;
   NSEnumerator        *rover;
   IMP                 impNextObject;
   IMP                 impAddObject;

   result = [NSMutableArray arrayWithCapacity:[lines count]];
   rover  = [lines reverseObjectEnumerator];

   impAddObject     = [result methodForSelector:@selector( addObject:)];
   impNextObject = [rover methodForSelector:@selector( nextObject)];

   while( s = impNextObject( rover, @selector( nextObject)))
      impAddObject( result, @selector( addObject:), s);
   
   return( result);
}


static NSMutableArray  *privateArray( NSArray *lines)
{
   PrivateMutableArray   *result;
   NSString              *s;
   NSEnumerator          *rover;
   IMP                   impNextObject;
   IMP                   impAddObject;

   result = [PrivateMutableArray arrayWithCapacity:[lines count]];
   rover  = [lines reverseObjectEnumerator];

   impAddObject  = [result methodForSelector:@selector( addObject:)];
   impNextObject = [rover methodForSelector:@selector( nextObject)];
   while( s = impNextObject( rover, @selector( nextObject)))
      impAddObject( result, @selector( addObject:), s);

   return( result);
}


static NSMutableArray  *finalCode( PrivateUnsafeMutableArray *lines)
{
   PrivateUnsafeMutableArray   *result;
   NSString                    *s;
   int                         i, n;

   n      = [lines count];
   result = [PrivateUnsafeMutableArray arrayWithCapacity:n];

   for( i = n; --i >= 0;)
   {
      s = PrivateUnsafeMutableArrayObjectAtIndex( lines, i);
      PrivateUnsafeMutableArrayAddObject( result, s);
   }

   return( result);
}


//
// Just a comparison for what we want to achieve ca...
//
static NSMutableArray  *straightC( NSArray *lines)
{
   id   *src, *dst;
   int  i, j, n;
   
   n   = [lines count];
   dst = malloc( sizeof( id) * n);
   src = malloc( sizeof( id) * n);
   
   for( i = n, j = 0; --i >= 0; j++)
      dst[ j] = src[ i];
   
   free( src);
   free( dst);
   return( nil); 
}


- (unsigned long) normalTestWithFunction:(NSMutableArray *(*)(NSArray *)) f
                               stopwatch:(Stopwatch *) watch
{
   NSAutoreleasePool     *pool2;
   NSArray               *lines;
   NSMutableArray        *result;
   NSString              *s;
   unsigned int           i, n;

   s      = [NSString stringWithContentsOfFile:@"/Developer/Documentation/ReleaseNotes/AppKitOlderNotes.html"];
   lines  = [s componentsSeparatedByString:@"\n"];
   n      = [lines count];

   pool2 = [[NSAutoreleasePool alloc] init];

   [watch start];
   for( i = 0; i < LOOPS; i++)
      result = (*f)( lines);
   [watch stop];

   [pool2 release];
   return( LOOPS * [lines count]);
}


- (unsigned long) privateStringTestWithFunction:(NSMutableArray *(*)(NSArray *)) f
                                      stopwatch:(Stopwatch *) watch
{
   NSAutoreleasePool     *pool2;
   NSArray               *lines;
   NSMutableArray        *result;
   NSString              *s;
   unsigned int           i, n;

   s      = [PrivateInlineRefCountingString stringWithContentsOfFile:TESTFILE];
   lines  = [s componentsSeparatedByString:@"\n"];
   n      = [lines count];

   pool2 = [[NSAutoreleasePool alloc] init];
   [watch start];
   for( i = 0; i < LOOPS; i++)
      result = (*f)( lines);
   [watch stop];
   
   [pool2 release];
   return( LOOPS * [lines count]);
}


- (unsigned long) finalEffortWithFunction:(NSMutableArray *(*)(NSArray *)) f
                                stopwatch:(Stopwatch *) watch
{
   NSAutoreleasePool           *pool2;
   NSArray                     *lines;
   PrivateUnsafeMutableArray   *privateLines;
   NSMutableArray              *result;
   NSString                    *s;
   unsigned int                i, n;

   s      = [PrivateInlineRefCountingString stringWithContentsOfFile:TESTFILE];
   lines  = [s componentsSeparatedByString:@"\n"];
   n      = [lines count];

   privateLines = [[[PrivateUnsafeMutableArray alloc] initWithArray:lines] autorelease];
   pool2 = [[NSAutoreleasePool alloc] init];
   [watch start];
   for( i = 0; i < LOOPS; i++)
      result = (*f)( privateLines);
   [watch stop];

   [pool2 release];
   return( LOOPS * [privateLines count]);
}


- (IBAction) straightC:(id)sender
{
   Stopwatch       *watch;
   unsigned long   loops;

   watch = [[[Stopwatch alloc] init] autorelease];
   loops = [self normalTestWithFunction:straightC
                              stopwatch:watch];
   [textView setString:[[textView string] stringByAppendingFormat:@"%@: %ld lines: %fs\n", [sender title],
      loops,
      [watch elapsedMilliseconds] / 1000.0]];
}


- (IBAction) privateArray:(id)sender
{
   Stopwatch       *watch;
   unsigned long   loops;

   watch = [[[Stopwatch alloc] init] autorelease];
   loops = [self normalTestWithFunction:privateArray
                              stopwatch:watch];
   [textView setString:[[textView string] stringByAppendingFormat:@"%@: %ld lines: %fs\n", [sender title],
      loops,
      [watch elapsedMilliseconds] / 1000.0]];
}


- (IBAction) privateString:(id)sender
{
   Stopwatch       *watch;
   unsigned long   loops;

   watch = [[[Stopwatch alloc] init] autorelease];
   loops = [self privateStringTestWithFunction:privateArray
                                     stopwatch:watch];
   [textView setString:[[textView string] stringByAppendingFormat:@"%@: %ld lines: %fs\n", [sender title],
      loops,
      [watch elapsedMilliseconds] / 1000.0]];
}


- (IBAction) finalEffort:(id)sender
{
   Stopwatch       *watch;
   unsigned long   loops;

   watch = [[[Stopwatch alloc] init] autorelease];
   loops = [self finalEffortWithFunction:finalCode
                               stopwatch:watch];
   [textView setString:[[textView string] stringByAppendingFormat:@"%@: %ld lines: %fs\n", [sender title],
      loops,
      [watch elapsedMilliseconds] / 1000.0]];
}


- (IBAction) unoptimized:(id)sender
{
   Stopwatch       *watch;
   unsigned long   loops;

   watch = [[[Stopwatch alloc] init] autorelease];
   loops = [self normalTestWithFunction:unoptimized
                              stopwatch:watch];
   [textView setString:[[textView string] stringByAppendingFormat:@"%@: %ld lines: %fs\n", [sender title],
      loops,
      [watch elapsedMilliseconds] / 1000.0]];
}


- (IBAction) imped:(id)sender
{
   Stopwatch       *watch;
   unsigned long   loops;
   
   watch = [[[Stopwatch alloc] init] autorelease];
   loops = [self normalTestWithFunction:imped
                              stopwatch:watch];
   [textView setString:[[textView string] stringByAppendingFormat:@"%@: %ld lines: %fs\n", [sender title],
      loops,
      [watch elapsedMilliseconds] / 1000.0]];

}




@end
