/*
 *  (C) Copyright 2001 Mulle kybernetiK. All rights reserved.
 *
 *  Permission to use, copy, modify and distribute this software and its
 *  documentation is hereby granted, provided that both the copyright notice
 *  and this permission notice appear in all copies of the software, derivative
 *  works or modified versions, and any portions thereof, and that both notices
 *  appear in supporting documentation, and that credit is given to
 *  Mulle kybernetiK in all documents and publicity pertaining to direct or
 *  indirect use of this code or its derivatives.
 *
 *  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH
 *  MAY HAVE SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS
 *  SOFTWARE IN ITS "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY
 *  LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING DIRECTLY OR
 *  INDIRECTLY FROM THE USE OF THIS SOFTWARE OR OF ANY DERIVATIVE WORK.
 *
 *  Coded by Nat!
 *
 *  $Id$
 *
 *  $Log$
 */
#import "PrivateMutableArray.h"



@implementation PrivateMutableArray

- (id) initWithCapacity:(unsigned int) n
{
   [super init];

   pointers_  = malloc( sizeof( id) * n);
   nPointers_ = 0;
   size_      = n;
   return( self);
}


- (id) initWithArray:(NSArray *) objects
{
   unsigned int  n;
   
   [super init];

   n = [objects count];
   
   pointers_  = malloc( sizeof( id) * n);
   nPointers_ = n;
   size_      = n;
   while( n)
   {
      --n;
      pointers_[ n] = [[objects objectAtIndex:n] retain];
   }
            
   return( self);
}


- (void) dealloc
{
   unsigned int   i;
   
   for( i = 0; i < nPointers_; i++)
      [pointers_[ i] release];

   free( pointers_);
   [super dealloc];
}


+ (id) arrayWithCapacity:(unsigned int) n
{
   return( [[[self alloc] initWithCapacity:n] autorelease]);
}


- (void) addObject:(id) p
{
   if( ! p)
      [NSException raise:NSInvalidArgumentException
                  format:@"null pointer"];
   if( self->nPointers_ >= self->size_)
      [NSException raise:NSInternalInconsistencyException
                  format:@"%@ can't grow", self];
#if USE_CF_RETAIN
   self->pointers_[ self->nPointers_++] = CFRetain( p);
#else
   self->pointers_[ self->nPointers_++] = [p retain];
#endif   
}


- (unsigned int) count
{
   return( nPointers_);
}


- (id) objectAtIndex:(unsigned int) i
{
   if( i >= nPointers_)
      [NSException raise:NSInvalidArgumentException
                  format:@"index %d out of bounds %d", i, nPointers_];
   return( pointers_[ i]);
}

@end
