/*
 *  (C) Copyright 2001 Mulle kybernetiK. All rights reserved.
 *
 *  Permission to use, copy, modify and distribute this software and its
 *  documentation is hereby granted, provided that both the copyright notice
 *  and this permission notice appear in all copies of the software, derivative
 *  works or modified versions, and any portions thereof, and that both notices
 *  appear in supporting documentation, and that credit is given to
 *  Mulle kybernetiK in all documents and publicity pertaining to direct or
 *  indirect use of this code or its derivatives.
 *
 *  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH
 *  MAY HAVE SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS
 *  SOFTWARE IN ITS "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY
 *  LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING DIRECTLY OR
 *  INDIRECTLY FROM THE USE OF THIS SOFTWARE OR OF ANY DERIVATIVE WORK.
 *
 *  Coded by Nat!
 *
 *  $Id$
 *
 *  $Log$
 */
#import "PrivateString.h"


@implementation PrivateString


//
// defeat NSPlaceholderString
//
+ (id) allocWithZone:(NSZone *) zone
{
   return( NSAllocateObject( self, 0, zone));
}


+ (id) alloc
{
   return( NSAllocateObject( self, 0, NULL));
}


- (void) dealloc
{
   free( string_);
   NSDeallocateObject( self);
}

// quickly coded...
- (id) initWithContentsOfFile:(NSString *) path
{
   NSString  *tmp;
   
   [super init];

   if( ! (tmp = [NSString stringWithContentsOfFile:path]))
   {
      [self release];
      return( nil);
   }
   len_    = [tmp length];
   string_ = malloc( sizeof( unichar) * len_);
   [tmp getCharacters:string_];

   return( self);
}


+ (id) stringWithContentsOfFile:(NSString *) path
{
   return( [[[self alloc] initWithContentsOfFile:path] autorelease]);
}


- (id) initWithCharacters:(const unichar *)characters
                   length:(unsigned)length
{
   [super init];

   len_    = length;
   string_ = malloc( sizeof( unichar) * len_);
   memcpy( string_, characters, sizeof( unichar) * len_);

   return( self);
}


- (id) initWithCString:(const char *) s
{
   unichar       *buf;
   unsigned int  i;
   
   [super init];

   len_    = strlen( s);
   string_ = buf = malloc( sizeof( unichar) * len_);
   for( i = 0; i < len_; i++)
      *buf++ = (unichar) *s++;

   return( self);
}


- (id) initWithString:(NSString *) s
{
   [super init];

   len_    = [s length];
   string_ = malloc( sizeof( unichar) * len_);
   [s getCharacters:string_];

   return( self);
}


- (unichar) characterAtIndex:(unsigned int) i
{
   if( i >= len_)
      [NSException raise:NSInvalidArgumentException
                  format:@"index %d out of bounds %d", i, len_];
   return( string_[ i]);
}


- (unsigned int) length
{
   return( len_);
}


- (NSString *) substringWithRange:(NSRange) range
{
   if( range.location >= len_)
      [NSException raise:NSInvalidArgumentException
                  format:@"index %d out of bounds %d", range.location, len_];
   return( [[[isa alloc] initWithCharacters:&string_[ range.location]
                                     length:range.length] autorelease]);
}


//
// this is probably a deoptimization. But we ain't measuring that.
//
- (NSArray *) componentsSeparatedByString:(NSString *) separator
{
   NSMutableArray   *array;
   NSRange          searchRange;
   NSRange          range;
   unsigned         seperatorLength;

   seperatorLength = [separator length];
   if( ! seperatorLength)
      return( [NSArray arrayWithObject:self]);
   
   array       = [NSMutableArray array];
   searchRange = NSMakeRange( 0, [self length]);
   while( searchRange.length)
   {
      range = [self rangeOfString:separator
                          options:NSLiteralSearch
                            range:searchRange];
      if( ! range.length)
      {
         [array addObject:[self substringWithRange:searchRange]];
         break;
      }
      [array addObject:[self substringWithRange:NSMakeRange( searchRange.location,
                                                             range.location - searchRange.location)]];
      searchRange.length  -= (range.location - searchRange.location) + seperatorLength;
      searchRange.location = range.location + seperatorLength;
   }
   return( array);
}


@end
